import { RQLQuery } from "../query/RQLQuery.js";

/**
 * Class representing a set of {@link RQLQuery} executed one after the other in a single HTTP request.
 *
 * These {@link RQLQuery} can be linked together by using a {@link RQLQueryCellRef}
 * of one {@link RQLQuery} in the parameters of a subsequent {@link RQLQuery}.
 *
 * Binary files are global to the transaction and can be referenced using a {@link RQLQueryBinaryRef}
 * in the parameters of a {@link RQLQuery}.
 */
export class Transaction {
  private _queries: Array<RQLQuery> = [];

  /**
   * Adds the given queries to the transaction.
   *
   * Each {@link RQLQuery} will be executed one after the other.
   * These {@link RQLQuery} can be linked together by using a {@link RQLQueryCellRef}
   * of one {@link RQLQuery} in the parameters of a subsequent {@link RQLQuery}.
   *
   * @param queries New queries to add to the transaction
   */
  push(...queries: RQLQuery[]): void {
    this._queries.push(...queries);
  }

  get queries(): ReadonlyArray<RQLQuery> {
    return this._queries;
  }
}
