# @cubicweb/react-form-utils

This library exposes react hooks and utilities to simplify writing React
clients for CubicWeb applications.

As the name suggests, those utilities focus primarily on building forms.
They build upon [`react-hook-form`](https://react-hook-form.com/) for form handling
and [`yup`](https://github.com/jquense/yup) for form validation.

## Installation

@cubicweb/react-form-utils is available from NPM:
```bash
# With NPM
npm i @cubicweb/react-form-utils
# Or with Yarn
yarn add @cubicweb/react-form-utils
```

## Usage

This library exports 4 main utilities:

- [useGetCubicWebEntity](https://cubicweb.pages.logilab.fr/react-form-utils/functions/useGetCubicWebEntity.html): Fetch an entity using its eid
- [useCubicWebForm](https://cubicweb.pages.logilab.fr/react-form-utils/functions/useCubicWebForm.html): Wrapper around [`react-hook-form`](https://react-hook-form.com/) and [`yup`](https://github.com/jquense/yup)
to connect your forms with CubicWeb
- [EntityContextProvider](https://cubicweb.pages.logilab.fr/react-form-utils/functions/EntityContextProvider.html): Provider and hooks to store the current entity schema and values
- [RelationDefinitionListContextProvider](https://cubicweb.pages.logilab.fr/react-form-utils/functions/RelationDefinitionListContextProvider.html): Provider and hooks to store the current relation definitions


## Documentation

You can find the full documentation with examples [here](https://cubicweb.pages.logilab.fr/cubicwebjs/react-form-utils/index.html).

## Notes on relations

For JS objects representing entity values returned by `useGetCubicWebEntity` and `useCubicWebForm`,
the relations are represented using the following format:

`SUBJECT__RELATION-TYPE__OBJECT`

So if you want to build a component handling a relation with several object types, you must be careful to
edit all the possible keys in the JS object.

This is a limitation from `@cubicweb/rql-generator`, while we are working on removing this behavior,
it is still a work in progress which may take time.


On the other hand, when passing relations to ignore in the `omittedFields` parameter, 
you use the relation type with an optional `reverse_` prefix
in case the current entity is object of the relation.

You never mention the subject/object. This is the behavior we want to achieve for the JS objects in the helpers.


## Contribute

All `@cubicweb` libraries are in the [cubicwebjs monorepo](https://forge.extranet.logilab.fr/cubicweb/cubicwebjs).
Please refer to the main README. 

## Get Help

Contact us on [Matrix](https://matrix.to/#/#cubicweb:matrix.logilab.org) and check the roadmap on the [CubicWeb Repository](https://forge.extranet.logilab.fr/cubicweb/cubicweb).